@@ -19,6 +19,7 @@ import System.IO
 
 import Command
 import Command.Checkout
+import Command.JobId
 import Command.Run
 import Config
 import Repo
@@ -73,6 +74,7 @@ commands :: NE.NonEmpty SomeCommandType
 commands =
     ( SC $ Proxy @RunCommand) NE.:|
     [ SC $ Proxy @CheckoutCommand
+    , SC $ Proxy @JobIdCommand
     ]
 
 lookupCommand :: String -> Maybe SomeCommandType
